/*
 *	@(#)led_tsk.c (usermain_t2ex)
 *
 *	Copyright (C) 2013 by Personal Media Corporation
 */
#include <basic.h>
#include <libstr.h>
#include <stdio.h>
#include <tk/syscall.h>
#include <tk/util.h>
#include <device/em1d512_iic.h>

#include "apd_cmd.h"

#define	RETRY		4
#define IIC2PEXPOAD	0x5c		/* ϥݡ IICɥ쥹       */
#define	DELAYTM		500

#define BIT_START	0x1
#define BIT_UP		0x2

LOCAL ID	flgid;			/* ٥ȥե饰ID		*/
LOCAL ID	tskid;			/* ID			*/

/*
	LED֤μ
	:bit7 -> LED8
               bit6 -> LED7
               bit5 -> LED6
               bit4 -> LED5
*/
LOCAL INT ledread()
{
	INT	er;
	INT	i;
	UH	cmddat[3];
	UH	tmp, led;

	cmddat[0] = IIC_START | (IIC2PEXPOAD << 1) | 0x01;
	cmddat[1] = IIC_RECV  | IIC_TOPDATA | IIC_LASTDATA;
	cmddat[2] = IIC_STOP;

	for (i = 0; i < RETRY; i++) {
		er = em1d512_iicxfer(1, cmddat, 3);
		if (er >= E_OK) break;
	}
	if (er < 0) goto err_ret;

	tmp = ~cmddat[1];
	led = ((tmp << 1) & 0x10) |
		 ((tmp << 3) & 0x20) |
		 ((tmp << 5) & 0x40) |
		 ((tmp << 7) & 0x80);
	return led;

err_ret:
	return er;
}

/*
	LED 
*/
LOCAL INT ledwrite(INT led)
{
	INT	er;
	INT	i;
	UH	cmddat[3];
	UH	tmp;

	led = ~led;
	tmp =	((led >> 1) & 0x08) |
		((led >> 3) & 0x04) |
		((led >> 5) & 0x02) |
		((led >> 7) & 0x01);
	cmddat[0] = IIC_START | (0x5c << 1) | 0x00;
	cmddat[1] = IIC_SEND  | IIC_TOPDATA | IIC_LASTDATA
			| tmp;
	cmddat[2] = IIC_STOP;
			
	for (i = 0; i < RETRY; i++) {
		er = em1d512_iicxfer(1, cmddat, 3);
		if (er >= E_OK) break;
	}
	return er;
}

/*
	LED 
*/
LOCAL void task_led(INT stacd, VP exinf)
{
	ER	er;
	UINT	flgptn;
	INT	led;
	
	for (;;) {
		er = tk_wai_flg(flgid, BIT_START, TWF_ORW, &flgptn, TMO_FEVR);
		if (er < E_OK) {
			break;
		}

		led = ledread();
		if (led < 0) break;

		if (flgptn & BIT_UP) {
			led <<= 1;
			if ((led & 0xf0) == 0)
				led = 0x10;
		} else {
			led >>= 1;
			if ((led & 0xf0) == 0)
				led = 0x80;
		}
		er = ledwrite(led);
		if (er < 0) break;

		er = tk_dly_tsk(DELAYTM);
		if (er != E_OK) {
			break;
		}
	}
	tk_exd_tsk();
}

/*
	֥Ȥ
*/
EXPORT ER mkledobjs(void)
{
	ER	er;
	T_CFLG	cflg;
	T_CTSK	ctsk;

	/* ٥ȥե饰 */
	memset(&cflg, 0, sizeof(cflg));

	cflg.flgatr = TA_TFIFO | TA_WSGL;/* ٥ȥե饰°		*/
	cflg.iflgptn = 0;		/* ٥ȥե饰ν 	*/

	flgid = tk_cre_flg(&cflg);
	if (flgid < 0){
		er = flgid;	goto err_ret;
	}

	/*  */
	memset(&ctsk, 0, sizeof(ctsk));

	SetOBJNAME(ctsk.exinf, "ledt");
	ctsk.tskatr = TA_HLNG | TA_RNG0;/* °			*/
	ctsk.task = task_led;		/* ưɥ쥹		*/
	ctsk.itskpri = 100;		/* ưͥ		*/
	ctsk.stksz = 8192;		/* 桼å(Х) */

	tskid = tk_cre_tsk(&ctsk);
	if (tskid < 0) {
		er = tskid;	goto err_ret;
	}

	/* εư */
	er = tk_sta_tsk(tskid, 0);
	if (er < E_OK)
		goto err_ret;

	return E_OK;

err_ret:
	if (tskid > 0) tk_del_tsk(tskid);
	if (flgid >= 0) tk_del_flg(flgid);
	return er;
}

/*
	LED Ǥư
*/
EXPORT void start(void)
{
	tk_set_flg(flgid, BIT_START);
}

/*
	LED Ǥư
*/
EXPORT void stop(void)
{
	tk_clr_flg(flgid, ~BIT_START);
}

/*
	LED Ǥ()
*/
EXPORT void up(void)
{
	tk_set_flg(flgid, BIT_UP);
}

/*
	LED Ǥ()
*/
EXPORT void down(void)
{
	tk_clr_flg(flgid, ~BIT_UP);
}

